@@ -44,7 +44,7 @@ pub enum CompatibleDist<'a> {
         /// The wheel that should be used.
         wheel: &'a RegistryBuiltWheel,
         /// The platform priority associated with the wheel.
-        priority: TagPriority,
+        priority: Option<TagPriority>,
         /// The prioritized distribution that the wheel came from.
         prioritized: &'a PrioritizedDist,
     },
@@ -145,7 +145,7 @@ pub enum PythonRequirementKind {
 #[derive(Debug, Clone, PartialEq, Eq)]
 pub enum WheelCompatibility {
     Incompatible(IncompatibleWheel),
-    Compatible(HashComparison, TagPriority, Option<BuildTag>),
+    Compatible(HashComparison, Option<TagPriority>, Option<BuildTag>),
 }
 
 #[derive(Debug, PartialEq, Eq, Clone)]
@@ -214,7 +214,7 @@ impl PrioritizedDist {
     pub fn insert_built(
         &mut self,
         dist: RegistryBuiltWheel,
-        hashes: Vec<HashDigest>,
+        hashes: impl IntoIterator<Item = HashDigest>,
         compatibility: WheelCompatibility,
     ) {
         // Track the highest-priority wheel.
@@ -233,7 +233,7 @@ impl PrioritizedDist {
     pub fn insert_source(
         &mut self,
         dist: RegistrySourceDist,
-        hashes: Vec<HashDigest>,
+        hashes: impl IntoIterator<Item = HashDigest>,
         compatibility: SourceDistCompatibility,
     ) {
         // Track the highest-priority source.
