@@ -295,9 +295,9 @@ impl Cache {
         // Add an empty .gitignore to the build bucket, to ensure that the cache's own .gitignore
         // doesn't interfere with source distribution builds. Build backends (like hatchling) will
         // traverse upwards to look for .gitignore files.
-        fs::create_dir_all(root.join(CacheBucket::BuiltWheels.to_str()))?;
+        fs::create_dir_all(root.join(CacheBucket::SourceDistributions.to_str()))?;
         match fs::OpenOptions::new().write(true).create_new(true).open(
-            root.join(CacheBucket::BuiltWheels.to_str())
+            root.join(CacheBucket::SourceDistributions.to_str())
                 .join(".gitignore"),
         ) {
             Ok(_) => {}
@@ -312,10 +312,10 @@ impl Cache {
         // We have to put this below the gitignore. Otherwise, if the build backend uses the rust
         // ignore crate it will walk up to the top level .gitignore and ignore its python source
         // files.
-        fs::OpenOptions::new()
-            .create(true)
-            .write(true)
-            .open(root.join(CacheBucket::BuiltWheels.to_str()).join(".git"))?;
+        fs::OpenOptions::new().create(true).write(true).open(
+            root.join(CacheBucket::SourceDistributions.to_str())
+                .join(".git"),
+        )?;
 
         Ok(Self {
             root: fs::canonicalize(root)?,
@@ -525,8 +525,8 @@ pub enum CacheBucket {
     ///                 └── flask-3.0.0.dist-info
     ///                     └── ...
     Wheels,
-    /// Wheels built from source distributions, their extracted metadata and the cache policy of
-    /// the source distribution.
+    /// Source distributions, wheels built from source distributions, their extracted metadata, and the
+    /// cache policy of the source distribution.
     ///
     /// The structure is similar of that of the `Wheel` bucket, except we have an additional layer
     /// for the source distribution filename and the metadata is at the source distribution-level,
@@ -535,6 +535,9 @@ pub enum CacheBucket {
     /// TODO(konstin): The cache policy should be on the source distribution level, the metadata we
     /// can put next to the wheels as in the `Wheels` bucket.
     ///
+    /// The unzipped source distribution is stored in a directory matching the source distribution
+    /// acrhive name.
+    ///
     /// Source distributions are built into zipped wheel files (as PEP 517 specifies) and unzipped
     /// lazily before installing. So when resolving, we only build the wheel and store the archive
     /// file in the cache, when installing, we unpack it under the same name (exclusive of the
@@ -566,32 +569,35 @@ pub enum CacheBucket {
     ///
     /// ...may be cached as:
     /// ```text
-    /// built-wheels-v0/
+    /// built-wheels-v3/
     /// ├── git
-    /// │   └── a67db8ed076e3814
-    /// │       └── 843b753e9e8cb74e83cac55598719b39a4d5ef1f
-    /// │           ├── manifest.msgpack
-    /// │           ├── metadata.msgpack
-    /// │           └── pydantic_extra_types-2.1.0-py3-none-any.whl
+    /// │   └── 2122faf3e081fb7a
+    /// │       └── 7a2d650a4a7b4d04
+    /// │           ├── metadata.msgpack
+    /// │           └── pydantic_extra_types-2.9.0-py3-none-any.whl
     /// ├── pypi
-    /// │   └── django
-    /// │       └── django-allauth-0.51.0.tar.gz
-    /// │           ├── django_allauth-0.51.0-py3-none-any.whl
-    /// │           ├── manifest.msgpack
-    /// │           └── metadata.msgpack
+    /// │   └── django-allauth
+    /// │       └── 0.51.0
+    /// │           ├── 0gH-_fwv8tdJ7JwwjJsUc
+    /// │           │   ├── django-allauth-0.51.0.tar.gz
+    /// │           │   │   └── [UNZIPPED CONTENTS]
+    /// │           │   ├── django_allauth-0.51.0-py3-none-any.whl
+    /// │           │   └── metadata.msgpack
+    /// │           └── revision.http
     /// └── url
     ///     └── 6781bd6440ae72c2
-    ///         └── werkzeug
-    ///             └── werkzeug-3.0.1.tar.gz
-    ///                 ├── manifest.msgpack
-    ///                 ├── metadata.msgpack
-    ///                 └── werkzeug-3.0.1-py3-none-any.whl
+    ///         ├── APYY01rbIfpAo_ij9sCY6
+    ///         │   ├── metadata.msgpack
+    ///         │   ├── werkzeug-3.0.1-py3-none-any.whl
+    ///         │   └── werkzeug-3.0.1.tar.gz
+    ///         │       └── [UNZIPPED CONTENTS]
+    ///         └── revision.http
     /// ```
     ///
     /// Structurally, the `manifest.msgpack` is empty, and only contains the caching information
     /// needed to invalidate the cache. The `metadata.msgpack` contains the metadata of the source
     /// distribution.
-    BuiltWheels,
+    SourceDistributions,
     /// Flat index responses, a format very similar to the simple metadata API.
     ///
     /// Cache structure:
@@ -663,7 +669,8 @@ pub enum CacheBucket {
 impl CacheBucket {
     fn to_str(self) -> &'static str {
         match self {
-            Self::BuiltWheels => "built-wheels-v3",
+            // Note, next time we change the version we should change the name of this bucket to `source-dists-v0`
+            Self::SourceDistributions => "built-wheels-v3",
             Self::FlatIndex => "flat-index-v0",
             Self::Git => "git-v0",
             Self::Interpreter => "interpreter-v2",
@@ -711,7 +718,7 @@ impl CacheBucket {
                     summary += rm_rf(directory.join(name.to_string()))?;
                 }
             }
-            Self::BuiltWheels => {
+            Self::SourceDistributions => {
                 // For `pypi` wheels, we expect a directory per package (indexed by name).
                 let root = cache.bucket(self).join(WheelCacheKind::Pypi);
                 summary += rm_rf(root.join(name.to_string()))?;
@@ -796,7 +803,7 @@ impl CacheBucket {
     pub fn iter() -> impl Iterator<Item = Self> {
         [
             Self::Wheels,
-            Self::BuiltWheels,
+            Self::SourceDistributions,
             Self::FlatIndex,
             Self::Git,
             Self::Interpreter,
