@@ -25,7 +25,7 @@ use uv_cache::{ArchiveId, ArchiveTimestamp, CacheBucket, CacheEntry, Timestamp,
 use uv_client::{
     CacheControl, CachedClientError, Connectivity, DataWithCachePolicy, RegistryClient,
 };
-use uv_configuration::{NoBinary, NoBuild, PreviewMode};
+use uv_configuration::PreviewMode;
 use uv_extract::hash::Hasher;
 use uv_fs::write_atomic;
 use uv_types::BuildContext;
@@ -145,13 +145,11 @@ impl<'a, Context: BuildContext> DistributionDatabase<'a, Context> {
         dist: &BuiltDist,
         hashes: HashPolicy<'_>,
     ) -> Result<LocalWheel, Error> {
-        let no_binary = match self.build_context.no_binary() {
-            NoBinary::None => false,
-            NoBinary::All => true,
-            NoBinary::Packages(packages) => packages.contains(dist.name()),
-        };
-
-        if no_binary {
+        if self
+            .build_context
+            .build_options()
+            .no_binary_package(dist.name())
+        {
             return Err(Error::NoBinary);
         }
 
@@ -162,16 +160,13 @@ impl<'a, Context: BuildContext> DistributionDatabase<'a, Context> {
                     FileLocation::RelativeUrl(base, url) => {
                         pypi_types::base_url_join_relative(base, url)?
                     }
-                    FileLocation::AbsoluteUrl(url) => {
-                        Url::parse(url).map_err(|err| Error::Url(url.clone(), err))?
-                    }
+                    FileLocation::AbsoluteUrl(url) => url.to_url(),
                     FileLocation::Path(path) => {
                         let cache_entry = self.build_context.cache().entry(
                             CacheBucket::Wheels,
                             WheelCache::Index(&wheel.index).wheel_dir(wheel.name().as_ref()),
                             wheel.filename.stem(),
                         );
-
                         return self
                             .load_wheel(path, &wheel.filename, cache_entry, dist, hashes)
                             .await;
@@ -185,6 +180,16 @@ impl<'a, Context: BuildContext> DistributionDatabase<'a, Context> {
                     wheel.filename.stem(),
                 );
 
+                // If the URL is a file URL, load the wheel directly.
+                if url.scheme() == "file" {
+                    let path = url
+                        .to_file_path()
+                        .map_err(|()| Error::NonFileUrl(url.clone()))?;
+                    return self
+                        .load_wheel(&path, &wheel.filename, wheel_entry, dist, hashes)
+                        .await;
+                }
+
                 // Download and unzip.
                 match self
                     .stream_wheel(
@@ -203,7 +208,7 @@ impl<'a, Context: BuildContext> DistributionDatabase<'a, Context> {
                         hashes: archive.hashes,
                         filename: wheel.filename.clone(),
                     }),
-                    Err(Error::Extract(err)) if err.is_http_streaming_unsupported() => {
+                    Err(Error::Client(err)) if err.is_http_streaming_unsupported() => {
                         warn!(
                             "Streaming unsupported for {dist}; downloading wheel to disk ({err})"
                         );
@@ -228,6 +233,37 @@ impl<'a, Context: BuildContext> DistributionDatabase<'a, Context> {
                             filename: wheel.filename.clone(),
                         })
                     }
+                    Err(Error::Extract(err)) => {
+                        if err.is_http_streaming_unsupported() {
+                            warn!(
+                                "Streaming unsupported for {dist}; downloading wheel to disk ({err})"
+                            );
+                        } else if err.is_http_streaming_failed() {
+                            warn!("Streaming failed for {dist}; downloading wheel to disk ({err})");
+                        } else {
+                            return Err(Error::Extract(err));
+                        }
+
+                        // If the request failed because streaming is unsupported, download the
+                        // wheel directly.
+                        let archive = self
+                            .download_wheel(
+                                url,
+                                &wheel.filename,
+                                wheel.file.size,
+                                &wheel_entry,
+                                dist,
+                                hashes,
+                            )
+                            .await?;
+
+                        Ok(LocalWheel {
+                            dist: Dist::Built(dist.clone()),
+                            archive: self.build_context.cache().archive(&archive.id),
+                            hashes: archive.hashes,
+                            filename: wheel.filename.clone(),
+                        })
+                    }
                     Err(err) => Err(err),
                 }
             }
@@ -282,6 +318,36 @@ impl<'a, Context: BuildContext> DistributionDatabase<'a, Context> {
                             filename: wheel.filename.clone(),
                         })
                     }
+                    Err(Error::Extract(err)) => {
+                        if err.is_http_streaming_unsupported() {
+                            warn!(
+                                "Streaming unsupported for {dist}; downloading wheel to disk ({err})"
+                            );
+                        } else if err.is_http_streaming_failed() {
+                            warn!("Streaming failed for {dist}; downloading wheel to disk ({err})");
+                        } else {
+                            return Err(Error::Extract(err));
+                        }
+
+                        // If the request failed because streaming is unsupported, download the
+                        // wheel directly.
+                        let archive = self
+                            .download_wheel(
+                                wheel.url.raw().clone(),
+                                &wheel.filename,
+                                None,
+                                &wheel_entry,
+                                dist,
+                                hashes,
+                            )
+                            .await?;
+                        Ok(LocalWheel {
+                            dist: Dist::Built(dist.clone()),
+                            archive: self.build_context.cache().archive(&archive.id),
+                            hashes: archive.hashes,
+                            filename: wheel.filename.clone(),
+                        })
+                    }
                     Err(err) => Err(err),
                 }
             }
@@ -379,11 +445,11 @@ impl<'a, Context: BuildContext> DistributionDatabase<'a, Context> {
 
         match result {
             Ok(metadata) => Ok(ArchiveMetadata::from_metadata23(metadata)),
-            Err(err) if err.is_http_streaming_unsupported() => {
-                warn!("Streaming unsupported when fetching metadata for {dist}; downloading wheel directly ({err})");
+            Err(err) if err.is_http_range_requests_unsupported() => {
+                warn!("Range requests unsupported when fetching metadata for {dist}; downloading wheel directly ({err})");
 
-                // If the request failed due to an error that could be resolved by
-                // downloading the wheel directly, try that.
+                // If the request failed due to an error that could be resolved by downloading the
+                // wheel directly, try that.
                 let wheel = self.get_wheel(dist, hashes).await?;
                 let metadata = wheel.metadata()?;
                 let hashes = wheel.hashes;
@@ -405,16 +471,12 @@ impl<'a, Context: BuildContext> DistributionDatabase<'a, Context> {
         source: &BuildableSource<'_>,
         hashes: HashPolicy<'_>,
     ) -> Result<ArchiveMetadata, Error> {
-        let no_build = match self.build_context.no_build() {
-            NoBuild::All => true,
-            NoBuild::None => false,
-            NoBuild::Packages(packages) => {
-                source.name().is_some_and(|name| packages.contains(name))
-            }
-        };
-
         // Optimization: Skip source dist download when we must not build them anyway.
-        if no_build {
+        if self
+            .build_context
+            .build_options()
+            .no_build_requirement(source.name())
+        {
             if source.is_editable() {
                 debug!("Allowing build for editable source distribution: {source}");
             } else {
@@ -452,14 +514,10 @@ impl<'a, Context: BuildContext> DistributionDatabase<'a, Context> {
         // Create an entry for the HTTP cache.
         let http_entry = wheel_entry.with_file(format!("{}.http", filename.stem()));
 
-        // Fetch the archive from the cache, or download it if necessary.
-        let req = self.request(url.clone())?;
-
-        // Extract the size from the `Content-Length` header, if not provided by the registry.
-        let size = size.or_else(|| content_length(&req));
-
         let download = |response: reqwest::Response| {
             async {
+                let size = size.or_else(|| content_length(&response));
+
                 let progress = self
                     .reporter
                     .as_ref()
@@ -516,6 +574,7 @@ impl<'a, Context: BuildContext> DistributionDatabase<'a, Context> {
 
         // Fetch the archive from the cache, or download it if necessary.
         let req = self.request(url.clone())?;
+
         let cache_control = match self.client.unmanaged.connectivity() {
             Connectivity::Online => CacheControl::from(
                 self.build_context
@@ -573,13 +632,10 @@ impl<'a, Context: BuildContext> DistributionDatabase<'a, Context> {
         // Create an entry for the HTTP cache.
         let http_entry = wheel_entry.with_file(format!("{}.http", filename.stem()));
 
-        let req = self.request(url.clone())?;
-
-        // Extract the size from the `Content-Length` header, if not provided by the registry.
-        let size = size.or_else(|| content_length(&req));
-
         let download = |response: reqwest::Response| {
             async {
+                let size = size.or_else(|| content_length(&response));
+
                 let progress = self
                     .reporter
                     .as_ref()
@@ -666,7 +722,9 @@ impl<'a, Context: BuildContext> DistributionDatabase<'a, Context> {
             .instrument(info_span!("wheel", wheel = %dist))
         };
 
+        // Fetch the archive from the cache, or download it if necessary.
         let req = self.request(url.clone())?;
+
         let cache_control = match self.client.unmanaged.connectivity() {
             Connectivity::Online => CacheControl::from(
                 self.build_context
@@ -886,9 +944,10 @@ impl<'a> ManagedClient<'a> {
     }
 }
 
-/// Returns the value of the `Content-Length` header from the [`reqwest::Request`], if present.
-fn content_length(req: &reqwest::Request) -> Option<u64> {
-    req.headers()
+/// Returns the value of the `Content-Length` header from the [`reqwest::Response`], if present.
+fn content_length(response: &reqwest::Response) -> Option<u64> {
+    response
+        .headers()
         .get(reqwest::header::CONTENT_LENGTH)
         .and_then(|val| val.to_str().ok())
         .and_then(|val| val.parse::<u64>().ok())
