@@ -7,15 +7,17 @@ pub use crate::discovery::{
 };
 pub use crate::environment::PythonEnvironment;
 pub use crate::implementation::ImplementationName;
-pub use crate::installation::PythonInstallation;
+pub use crate::installation::{PythonInstallation, PythonInstallationKey};
 pub use crate::interpreter::{Error as InterpreterError, Interpreter};
 pub use crate::pointer_size::PointerSize;
 pub use crate::prefix::Prefix;
 pub use crate::python_version::PythonVersion;
 pub use crate::target::Target;
-pub use crate::version_files::{request_from_version_file, requests_from_version_file};
+pub use crate::version_files::{
+    request_from_version_file, requests_from_version_file, version_file_exists,
+    versions_file_exists, PYTHON_VERSIONS_FILENAME, PYTHON_VERSION_FILENAME,
+};
 pub use crate::virtualenv::{Error as VirtualEnvError, PyVenvConfiguration, VirtualEnvironment};
-
 mod discovery;
 pub mod downloads;
 mod environment;
