@@ -9,13 +9,13 @@ use install_wheel_rs::linker::LinkMode;
 use pep508_rs::{ExtraName, RequirementOrigin};
 use pypi_types::Requirement;
 use uv_cache::{CacheArgs, Refresh};
-use uv_cli::options::{flag, installer_options, resolver_installer_options, resolver_options};
+use uv_cli::options::{flag, resolver_installer_options, resolver_options};
 use uv_cli::{
     AddArgs, ColorChoice, Commands, ExternalCommand, GlobalArgs, ListFormat, LockArgs, Maybe,
     PipCheckArgs, PipCompileArgs, PipFreezeArgs, PipInstallArgs, PipListArgs, PipShowArgs,
-    PipSyncArgs, PipTreeArgs, PipUninstallArgs, RemoveArgs, RunArgs, SyncArgs, ToolInstallArgs,
-    ToolListArgs, ToolRunArgs, ToolUninstallArgs, ToolchainFindArgs, ToolchainInstallArgs,
-    ToolchainListArgs, ToolchainUninstallArgs, VenvArgs,
+    PipSyncArgs, PipTreeArgs, PipUninstallArgs, PythonFindArgs, PythonInstallArgs, PythonListArgs,
+    PythonPinArgs, PythonUninstallArgs, RemoveArgs, RunArgs, SyncArgs, ToolInstallArgs,
+    ToolListArgs, ToolRunArgs, ToolUninstallArgs, TreeArgs, VenvArgs,
 };
 use uv_client::Connectivity;
 use uv_configuration::{
@@ -25,13 +25,12 @@ use uv_configuration::{
 };
 use uv_distribution::pyproject::DependencyType;
 use uv_normalize::PackageName;
+use uv_python::{Prefix, PythonFetch, PythonPreference, PythonVersion, Target};
 use uv_requirements::RequirementsSource;
 use uv_resolver::{AnnotationStyle, DependencyMode, ExcludeNewer, PreReleaseMode, ResolutionMode};
 use uv_settings::{
-    Combine, FilesystemOptions, InstallerOptions, Options, PipOptions, ResolverInstallerOptions,
-    ResolverOptions,
+    Combine, FilesystemOptions, Options, PipOptions, ResolverInstallerOptions, ResolverOptions,
 };
-use uv_toolchain::{Prefix, PythonVersion, Target, ToolchainFetch, ToolchainPreference};
 
 use crate::commands::pip::operations::Modifications;
 
@@ -47,8 +46,9 @@ pub(crate) struct GlobalSettings {
     pub(crate) isolated: bool,
     pub(crate) show_settings: bool,
     pub(crate) preview: PreviewMode,
-    pub(crate) toolchain_preference: ToolchainPreference,
-    pub(crate) toolchain_fetch: ToolchainFetch,
+    pub(crate) python_preference: PythonPreference,
+    pub(crate) python_fetch: PythonFetch,
+    pub(crate) no_progress: bool,
 }
 
 impl GlobalSettings {
@@ -64,17 +64,17 @@ impl GlobalSettings {
                 .unwrap_or(false),
         );
 
-        // Always use preview mode toolchain preferences during preview commands
+        // Always use preview mode python preferences during preview commands
         // TODO(zanieb): There should be a cleaner way to do this, we should probably resolve
         // force preview to true for these commands but it would break our experimental warning
         // right now
-        let default_toolchain_preference = if matches!(
+        let default_python_preference = if matches!(
             command,
-            Commands::Project(_) | Commands::Toolchain(_) | Commands::Tool(_)
+            Commands::Project(_) | Commands::Python(_) | Commands::Tool(_)
         ) {
-            ToolchainPreference::default_from(PreviewMode::Enabled)
+            PythonPreference::default_from(PreviewMode::Enabled)
         } else {
-            ToolchainPreference::default_from(preview)
+            PythonPreference::default_from(preview)
         };
 
         Self {
@@ -111,14 +111,15 @@ impl GlobalSettings {
             isolated: args.isolated,
             show_settings: args.show_settings,
             preview,
-            toolchain_preference: args
-                .toolchain_preference
-                .combine(workspace.and_then(|workspace| workspace.globals.toolchain_preference))
-                .unwrap_or(default_toolchain_preference),
-            toolchain_fetch: args
-                .toolchain_fetch
-                .combine(workspace.and_then(|workspace| workspace.globals.toolchain_fetch))
+            python_preference: args
+                .python_preference
+                .combine(workspace.and_then(|workspace| workspace.globals.python_preference))
+                .unwrap_or(default_python_preference),
+            python_fetch: args
+                .python_fetch
+                .combine(workspace.and_then(|workspace| workspace.globals.python_fetch))
                 .unwrap_or_default(),
+            no_progress: args.no_progress,
         }
     }
 }
@@ -300,21 +301,23 @@ impl ToolListSettings {
 #[allow(clippy::struct_excessive_bools)]
 #[derive(Debug, Clone)]
 pub(crate) struct ToolUninstallSettings {
-    pub(crate) name: PackageName,
+    pub(crate) name: Option<PackageName>,
 }
 
 impl ToolUninstallSettings {
     /// Resolve the [`ToolUninstallSettings`] from the CLI and filesystem configuration.
     #[allow(clippy::needless_pass_by_value)]
     pub(crate) fn resolve(args: ToolUninstallArgs, _filesystem: Option<FilesystemOptions>) -> Self {
-        let ToolUninstallArgs { name } = args;
+        let ToolUninstallArgs { name, all } = args;
 
-        Self { name }
+        Self {
+            name: name.filter(|_| !all),
+        }
     }
 }
 
 #[derive(Debug, Clone, Default)]
-pub(crate) enum ToolchainListKinds {
+pub(crate) enum PythonListKinds {
     #[default]
     Default,
     Installed,
@@ -323,26 +326,26 @@ pub(crate) enum ToolchainListKinds {
 /// The resolved settings to use for a `tool run` invocation.
 #[allow(clippy::struct_excessive_bools)]
 #[derive(Debug, Clone)]
-pub(crate) struct ToolchainListSettings {
-    pub(crate) kinds: ToolchainListKinds,
+pub(crate) struct PythonListSettings {
+    pub(crate) kinds: PythonListKinds,
     pub(crate) all_platforms: bool,
     pub(crate) all_versions: bool,
 }
 
-impl ToolchainListSettings {
-    /// Resolve the [`ToolchainListSettings`] from the CLI and filesystem configuration.
+impl PythonListSettings {
+    /// Resolve the [`PythonListSettings`] from the CLI and filesystem configuration.
     #[allow(clippy::needless_pass_by_value)]
-    pub(crate) fn resolve(args: ToolchainListArgs, _filesystem: Option<FilesystemOptions>) -> Self {
-        let ToolchainListArgs {
+    pub(crate) fn resolve(args: PythonListArgs, _filesystem: Option<FilesystemOptions>) -> Self {
+        let PythonListArgs {
             all_versions,
             all_platforms,
             only_installed,
         } = args;
 
         let kinds = if only_installed {
-            ToolchainListKinds::Installed
+            PythonListKinds::Installed
         } else {
-            ToolchainListKinds::default()
+            PythonListKinds::default()
         };
 
         Self {
@@ -353,74 +356,98 @@ impl ToolchainListSettings {
     }
 }
 
-/// The resolved settings to use for a `toolchain install` invocation.
+/// The resolved settings to use for a `python install` invocation.
 #[allow(clippy::struct_excessive_bools)]
 #[derive(Debug, Clone)]
-pub(crate) struct ToolchainInstallSettings {
+pub(crate) struct PythonInstallSettings {
     pub(crate) targets: Vec<String>,
-    pub(crate) force: bool,
+    pub(crate) reinstall: bool,
 }
 
-impl ToolchainInstallSettings {
-    /// Resolve the [`ToolchainInstallSettings`] from the CLI and filesystem configuration.
+impl PythonInstallSettings {
+    /// Resolve the [`PythonInstallSettings`] from the CLI and filesystem configuration.
     #[allow(clippy::needless_pass_by_value)]
-    pub(crate) fn resolve(
-        args: ToolchainInstallArgs,
-        _filesystem: Option<FilesystemOptions>,
-    ) -> Self {
-        let ToolchainInstallArgs { targets, force } = args;
+    pub(crate) fn resolve(args: PythonInstallArgs, _filesystem: Option<FilesystemOptions>) -> Self {
+        let PythonInstallArgs { targets, reinstall } = args;
 
-        Self { targets, force }
+        Self { targets, reinstall }
     }
 }
 
-/// The resolved settings to use for a `toolchain uninstall` invocation.
+/// The resolved settings to use for a `python uninstall` invocation.
 #[allow(clippy::struct_excessive_bools)]
 #[derive(Debug, Clone)]
-pub(crate) struct ToolchainUninstallSettings {
+pub(crate) struct PythonUninstallSettings {
     pub(crate) targets: Vec<String>,
+    pub(crate) all: bool,
 }
 
-impl ToolchainUninstallSettings {
-    /// Resolve the [`ToolchainUninstallSettings`] from the CLI and filesystem configuration.
+impl PythonUninstallSettings {
+    /// Resolve the [`PythonUninstallSettings`] from the CLI and filesystem configuration.
     #[allow(clippy::needless_pass_by_value)]
     pub(crate) fn resolve(
-        args: ToolchainUninstallArgs,
+        args: PythonUninstallArgs,
         _filesystem: Option<FilesystemOptions>,
     ) -> Self {
-        let ToolchainUninstallArgs { targets } = args;
+        let PythonUninstallArgs { targets, all } = args;
 
-        Self { targets }
+        Self { targets, all }
     }
 }
 
-/// The resolved settings to use for a `toolchain find` invocation.
+/// The resolved settings to use for a `python find` invocation.
 #[allow(clippy::struct_excessive_bools)]
 #[derive(Debug, Clone)]
-pub(crate) struct ToolchainFindSettings {
+pub(crate) struct PythonFindSettings {
     pub(crate) request: Option<String>,
 }
 
-impl ToolchainFindSettings {
-    /// Resolve the [`ToolchainFindSettings`] from the CLI and workspace configuration.
+impl PythonFindSettings {
+    /// Resolve the [`PythonFindSettings`] from the CLI and workspace configuration.
     #[allow(clippy::needless_pass_by_value)]
-    pub(crate) fn resolve(args: ToolchainFindArgs, _filesystem: Option<FilesystemOptions>) -> Self {
-        let ToolchainFindArgs { request } = args;
+    pub(crate) fn resolve(args: PythonFindArgs, _filesystem: Option<FilesystemOptions>) -> Self {
+        let PythonFindArgs { request } = args;
 
         Self { request }
     }
 }
 
+/// The resolved settings to use for a `python pin` invocation.
+#[allow(clippy::struct_excessive_bools)]
+#[derive(Debug, Clone)]
+pub(crate) struct PythonPinSettings {
+    pub(crate) request: Option<String>,
+    pub(crate) resolved: bool,
+}
+
+impl PythonPinSettings {
+    /// Resolve the [`PythonPinSettings`] from the CLI and workspace configuration.
+    #[allow(clippy::needless_pass_by_value)]
+    pub(crate) fn resolve(args: PythonPinArgs, _filesystem: Option<FilesystemOptions>) -> Self {
+        let PythonPinArgs {
+            request,
+            no_resolved,
+            resolved,
+        } = args;
+
+        Self {
+            request,
+            resolved: flag(resolved, no_resolved).unwrap_or(false),
+        }
+    }
+}
 /// The resolved settings to use for a `sync` invocation.
 #[allow(clippy::struct_excessive_bools, dead_code)]
 #[derive(Debug, Clone)]
 pub(crate) struct SyncSettings {
+    pub(crate) locked: bool,
+    pub(crate) frozen: bool,
     pub(crate) extras: ExtrasSpecification,
     pub(crate) dev: bool,
     pub(crate) modifications: Modifications,
     pub(crate) python: Option<String>,
     pub(crate) refresh: Refresh,
-    pub(crate) settings: InstallerSettings,
+    pub(crate) settings: ResolverInstallerSettings,
 }
 
 impl SyncSettings {
@@ -428,6 +455,8 @@ impl SyncSettings {
     #[allow(clippy::needless_pass_by_value)]
     pub(crate) fn resolve(args: SyncArgs, filesystem: Option<FilesystemOptions>) -> Self {
         let SyncArgs {
+            locked,
+            frozen,
             extra,
             all_extras,
             no_all_extras,
@@ -447,6 +476,8 @@ impl SyncSettings {
         };
 
         Self {
+            locked,
+            frozen,
             extras: ExtrasSpecification::from_args(
                 flag(all_extras, no_all_extras).unwrap_or_default(),
                 extra.unwrap_or_default(),
@@ -455,7 +486,10 @@ impl SyncSettings {
             modifications,
             python,
             refresh: Refresh::from(refresh),
-            settings: InstallerSettings::combine(installer_options(installer, build), filesystem),
+            settings: ResolverInstallerSettings::combine(
+                resolver_installer_options(installer, build),
+                filesystem,
+            ),
         }
     }
 }
@@ -568,16 +602,21 @@ pub(crate) struct RemoveSettings {
     pub(crate) dependency_type: DependencyType,
     pub(crate) package: Option<PackageName>,
     pub(crate) python: Option<String>,
+    pub(crate) refresh: Refresh,
+    pub(crate) settings: ResolverInstallerSettings,
 }
 
 impl RemoveSettings {
     /// Resolve the [`RemoveSettings`] from the CLI and filesystem configuration.
     #[allow(clippy::needless_pass_by_value)]
-    pub(crate) fn resolve(args: RemoveArgs, _filesystem: Option<FilesystemOptions>) -> Self {
+    pub(crate) fn resolve(args: RemoveArgs, filesystem: Option<FilesystemOptions>) -> Self {
         let RemoveArgs {
             dev,
             optional,
             requirements,
+            installer,
+            build,
+            refresh,
             package,
             python,
         } = args;
@@ -595,10 +634,49 @@ impl RemoveSettings {
             dependency_type,
             package,
             python,
+            refresh: Refresh::from(refresh),
+            settings: ResolverInstallerSettings::combine(
+                resolver_installer_options(installer, build),
+                filesystem,
+            ),
         }
     }
 }
 
+/// The resolved settings to use for a `tree` invocation.
+#[allow(clippy::struct_excessive_bools)]
+#[derive(Debug, Clone)]
+pub(crate) struct TreeSettings {
+    pub(crate) depth: u8,
+    pub(crate) prune: Vec<PackageName>,
+    pub(crate) package: Vec<PackageName>,
+    pub(crate) no_dedupe: bool,
+    pub(crate) invert: bool,
+    pub(crate) python: Option<String>,
+    pub(crate) resolver: ResolverSettings,
+}
+
+impl TreeSettings {
+    /// Resolve the [`TreeSettings`] from the CLI and workspace configuration.
+    pub(crate) fn resolve(args: TreeArgs, filesystem: Option<FilesystemOptions>) -> Self {
+        let TreeArgs {
+            tree,
+            build,
+            resolver,
+            python,
+        } = args;
+
+        Self {
+            python,
+            depth: tree.depth,
+            prune: tree.prune,
+            package: tree.package,
+            no_dedupe: tree.no_dedupe,
+            invert: tree.invert,
+            resolver: ResolverSettings::combine(resolver_options(resolver, build), filesystem),
+        }
+    }
+}
 /// The resolved settings to use for a `pip compile` invocation.
 #[allow(clippy::struct_excessive_bools)]
 #[derive(Debug, Clone)]
@@ -687,7 +765,10 @@ impl PipCompileSettings {
                 .into_iter()
                 .filter_map(Maybe::into_option)
                 .collect(),
-            r#override,
+            r#override: r#override
+                .into_iter()
+                .filter_map(Maybe::into_option)
+                .collect(),
             overrides_from_workspace,
             refresh: Refresh::from(refresh),
             settings: PipSettings::combine(
@@ -743,13 +824,12 @@ pub(crate) struct PipSyncSettings {
 
 impl PipSyncSettings {
     /// Resolve the [`PipSyncSettings`] from the CLI and filesystem configuration.
-    pub(crate) fn resolve(args: PipSyncArgs, filesystem: Option<FilesystemOptions>) -> Self {
+    pub(crate) fn resolve(args: Box<PipSyncArgs>, filesystem: Option<FilesystemOptions>) -> Self {
         let PipSyncArgs {
             src_file,
             constraint,
             installer,
             refresh,
-            exclude_newer,
             require_hashes,
             no_require_hashes,
             python,
@@ -775,7 +855,7 @@ impl PipSyncSettings {
             no_strict,
             dry_run,
             compat_args: _,
-        } = args;
+        } = *args;
 
         Self {
             src_file,
@@ -790,7 +870,6 @@ impl PipSyncSettings {
                     python,
                     system: flag(system, no_system),
                     break_system_packages: flag(break_system_packages, no_break_system_packages),
-                    exclude_newer,
                     target,
                     prefix,
                     require_hashes: flag(require_hashes, no_require_hashes),
@@ -895,7 +974,10 @@ impl PipInstallSettings {
                 .into_iter()
                 .filter_map(Maybe::into_option)
                 .collect(),
-            r#override,
+            r#override: r#override
+                .into_iter()
+                .filter_map(Maybe::into_option)
+                .collect(),
             dry_run,
             overrides_from_workspace,
             refresh: Refresh::from(refresh),
@@ -1091,7 +1173,7 @@ impl PipShowSettings {
     }
 }
 
-/// The resolved settings to use for a `pip show` invocation.
+/// The resolved settings to use for a `pip tree` invocation.
 #[allow(clippy::struct_excessive_bools)]
 #[derive(Debug, Clone)]
 pub(crate) struct PipTreeSettings {
@@ -1108,11 +1190,7 @@ impl PipTreeSettings {
     /// Resolve the [`PipTreeSettings`] from the CLI and workspace configuration.
     pub(crate) fn resolve(args: PipTreeArgs, filesystem: Option<FilesystemOptions>) -> Self {
         let PipTreeArgs {
-            depth,
-            prune,
-            package,
-            no_dedupe,
-            invert,
+            tree,
             strict,
             no_strict,
             python,
@@ -1122,11 +1200,11 @@ impl PipTreeSettings {
         } = args;
 
         Self {
-            depth,
-            prune,
-            package,
-            no_dedupe,
-            invert,
+            depth: tree.depth,
+            prune: tree.prune,
+            no_dedupe: tree.no_dedupe,
+            invert: tree.invert,
+            package: tree.package,
             // Shared settings.
             shared: PipSettings::combine(
                 PipOptions {
@@ -1224,128 +1302,24 @@ impl VenvSettings {
     }
 }
 
-/// The resolved settings to use for an invocation of the `uv` CLI when installing dependencies.
+/// The resolved settings to use for an invocation of the uv CLI when installing dependencies.
 ///
 /// Combines the `[tool.uv]` persistent configuration with the command-line arguments
 /// ([`InstallerArgs`], represented as [`InstallerOptions`]).
-#[allow(clippy::struct_excessive_bools)]
-#[derive(Debug, Clone, Default)]
-pub(crate) struct InstallerSettings {
-    pub(crate) index_locations: IndexLocations,
-    pub(crate) index_strategy: IndexStrategy,
-    pub(crate) keyring_provider: KeyringProviderType,
-    pub(crate) config_setting: ConfigSettings,
-    pub(crate) link_mode: LinkMode,
-    pub(crate) compile_bytecode: bool,
-    pub(crate) reinstall: Reinstall,
-    pub(crate) build_options: BuildOptions,
-}
-
 #[derive(Debug, Clone)]
 pub(crate) struct InstallerSettingsRef<'a> {
     pub(crate) index_locations: &'a IndexLocations,
     pub(crate) index_strategy: IndexStrategy,
     pub(crate) keyring_provider: KeyringProviderType,
     pub(crate) config_setting: &'a ConfigSettings,
+    pub(crate) exclude_newer: Option<ExcludeNewer>,
     pub(crate) link_mode: LinkMode,
     pub(crate) compile_bytecode: bool,
     pub(crate) reinstall: &'a Reinstall,
     pub(crate) build_options: &'a BuildOptions,
 }
 
-impl InstallerSettings {
-    /// Resolve the [`InstallerSettings`] from the CLI and filesystem configuration.
-    pub(crate) fn combine(args: InstallerOptions, filesystem: Option<FilesystemOptions>) -> Self {
-        let ResolverInstallerOptions {
-            index_url,
-            extra_index_url,
-            no_index,
-            find_links,
-            index_strategy,
-            keyring_provider,
-            resolution: _,
-            prerelease: _,
-            config_settings,
-            exclude_newer: _,
-            link_mode,
-            compile_bytecode,
-            upgrade: _,
-            upgrade_package: _,
-            reinstall,
-            reinstall_package,
-            no_build,
-            no_build_package,
-            no_binary,
-            no_binary_package,
-        } = filesystem
-            .map(FilesystemOptions::into_options)
-            .map(|options| options.top_level)
-            .unwrap_or_default();
-
-        Self {
-            index_locations: IndexLocations::new(
-                args.index_url.combine(index_url),
-                args.extra_index_url
-                    .combine(extra_index_url)
-                    .unwrap_or_default(),
-                args.find_links.combine(find_links).unwrap_or_default(),
-                args.no_index.combine(no_index).unwrap_or_default(),
-            ),
-            index_strategy: args
-                .index_strategy
-                .combine(index_strategy)
-                .unwrap_or_default(),
-            keyring_provider: args
-                .keyring_provider
-                .combine(keyring_provider)
-                .unwrap_or_default(),
-            config_setting: args
-                .config_settings
-                .combine(config_settings)
-                .unwrap_or_default(),
-            link_mode: args.link_mode.combine(link_mode).unwrap_or_default(),
-            compile_bytecode: args
-                .compile_bytecode
-                .combine(compile_bytecode)
-                .unwrap_or_default(),
-            reinstall: Reinstall::from_args(
-                args.reinstall.combine(reinstall),
-                args.reinstall_package
-                    .combine(reinstall_package)
-                    .unwrap_or_default(),
-            ),
-            build_options: BuildOptions::new(
-                NoBinary::from_args(
-                    args.no_binary.combine(no_binary),
-                    args.no_binary_package
-                        .combine(no_binary_package)
-                        .unwrap_or_default(),
-                ),
-                NoBuild::from_args(
-                    args.no_build.combine(no_build),
-                    args.no_build_package
-                        .combine(no_build_package)
-                        .unwrap_or_default(),
-                ),
-            ),
-        }
-    }
-
-    pub(crate) fn as_ref(&self) -> InstallerSettingsRef {
-        InstallerSettingsRef {
-            index_locations: &self.index_locations,
-            index_strategy: self.index_strategy,
-            keyring_provider: self.keyring_provider,
-            config_setting: &self.config_setting,
-            link_mode: self.link_mode,
-            compile_bytecode: self.compile_bytecode,
-            reinstall: &self.reinstall,
-            build_options: &self.build_options,
-        }
-    }
-}
-
-/// The resolved settings to use for an invocation of the `uv` CLI when resolving dependencies.
+/// The resolved settings to use for an invocation of the uv CLI when resolving dependencies.
 ///
 /// Combines the `[tool.uv]` persistent configuration with the command-line arguments
 /// ([`ResolverArgs`], represented as [`ResolverOptions`]).
@@ -1436,7 +1410,10 @@ impl ResolverSettings {
                 args.upgrade.combine(upgrade),
                 args.upgrade_package
                     .combine(upgrade_package)
-                    .unwrap_or_default(),
+                    .into_iter()
+                    .flatten()
+                    .map(Requirement::from)
+                    .collect(),
             ),
             build_options: BuildOptions::new(
                 NoBinary::from_args(
@@ -1471,10 +1448,10 @@ impl ResolverSettings {
     }
 }
 
-/// The resolved settings to use for an invocation of the `uv` CLI with both resolver and installer
+/// The resolved settings to use for an invocation of the uv CLI with both resolver and installer
 /// capabilities.
 ///
-/// Represents the shared settings that are used across all `uv` commands outside the `pip` API.
+/// Represents the shared settings that are used across all uv commands outside the `pip` API.
 /// Analogous to the settings contained in the `[tool.uv]` table, combined with [`ResolverInstallerArgs`].
 #[allow(clippy::struct_excessive_bools)]
 #[derive(Debug, Clone, Default)]
@@ -1574,7 +1551,10 @@ impl ResolverInstallerSettings {
                 args.upgrade.combine(upgrade),
                 args.upgrade_package
                     .combine(upgrade_package)
-                    .unwrap_or_default(),
+                    .into_iter()
+                    .flatten()
+                    .map(Requirement::from)
+                    .collect(),
             ),
             reinstall: Reinstall::from_args(
                 args.reinstall.combine(reinstall),
@@ -1893,7 +1873,10 @@ impl PipSettings {
                 args.upgrade.combine(upgrade),
                 args.upgrade_package
                     .combine(upgrade_package)
-                    .unwrap_or_default(),
+                    .into_iter()
+                    .flatten()
+                    .map(Requirement::from)
+                    .collect(),
             ),
             reinstall: Reinstall::from_args(
                 args.reinstall.combine(reinstall),
@@ -1961,6 +1944,7 @@ impl<'a> From<ResolverInstallerSettingsRef<'a>> for InstallerSettingsRef<'a> {
             index_strategy: settings.index_strategy,
             keyring_provider: settings.keyring_provider,
             config_setting: settings.config_setting,
+            exclude_newer: settings.exclude_newer,
             link_mode: settings.link_mode,
             compile_bytecode: settings.compile_bytecode,
             reinstall: settings.reinstall,
