@@ -1,11 +1,16 @@
+use std::collections::BTreeMap;
 use std::env;
 use std::path::{Path, PathBuf};
 use std::process::Command;
 
 use anyhow::Result;
 use assert_cmd::assert::OutputAssertExt;
+use assert_fs::fixture::{FileWriteStr, PathChild};
+use indoc::indoc;
+use insta::assert_json_snapshot;
+use serde::{Deserialize, Serialize};
 
-use crate::common::{copy_dir_ignore, uv_snapshot, TestContext};
+use crate::common::{copy_dir_ignore, make_project, uv_snapshot, TestContext};
 
 mod common;
 
@@ -167,6 +172,21 @@ fn test_albatross_project_in_excluded() {
     );
 
     context.assert_file(current_dir.join("check_installed_bird_feeder.py"));
+
+    let current_dir = workspaces_dir()
+        .join("albatross-project-in-excluded")
+        .join("packages")
+        .join("seeds");
+    uv_snapshot!(context.filters(), install_workspace(&context, &current_dir), @r###"
+    success: false
+    exit_code: 2
+    ----- stdout -----
+
+    ----- stderr -----
+    error: Failed to download and build: `seeds @ file://[WORKSPACE]/scripts/workspaces/albatross-project-in-excluded/packages/seeds`
+      Caused by: The project is marked as unmanaged: `[WORKSPACE]/scripts/workspaces/albatross-project-in-excluded/packages/seeds`
+    "###
+    );
 }
 
 #[test]
@@ -341,6 +361,7 @@ fn test_uv_run_with_package_virtual_workspace() -> Result<()> {
         "Using Python 3.12.[X] interpreter at: [PYTHON]",
     ));
 
+    // Run from the `bird-feeder` member.
     uv_snapshot!(filters, context
         .run()
         .arg("--preview")
@@ -370,10 +391,10 @@ fn test_uv_run_with_package_virtual_workspace() -> Result<()> {
     uv_snapshot!(context.filters(), universal_windows_filters=true, context
         .run()
         .arg("--preview")
-            .arg("--package")
-            .arg("albatross")
-            .arg("packages/albatross/check_installed_albatross.py")
-            .current_dir(&work_dir), @r###"
+        .arg("--package")
+        .arg("albatross")
+        .arg("packages/albatross/check_installed_albatross.py")
+        .current_dir(&work_dir), @r###"
     success: true
     exit_code: 0
     ----- stdout -----
@@ -391,6 +412,47 @@ fn test_uv_run_with_package_virtual_workspace() -> Result<()> {
     Ok(())
 }
 
+/// Check that `uv run` works from a virtual workspace root, which should sync all packages in the
+/// workspace.
+#[test]
+fn test_uv_run_virtual_workspace_root() -> Result<()> {
+    let context = TestContext::new("3.12");
+    let work_dir = context.temp_dir.join("albatross-virtual-workspace");
+
+    copy_dir_ignore(
+        workspaces_dir().join("albatross-virtual-workspace"),
+        &work_dir,
+    )?;
+
+    uv_snapshot!(context.filters(), universal_windows_filters=true, context
+        .run()
+        .arg("--preview")
+        .arg("packages/albatross/check_installed_albatross.py")
+        .current_dir(&work_dir), @r###"
+    success: true
+    exit_code: 0
+    ----- stdout -----
+    Success
+
+    ----- stderr -----
+    Using Python 3.12.[X] interpreter at: [PYTHON-3.12]
+    Creating virtualenv at: .venv
+    Resolved 8 packages in [TIME]
+    Prepared 7 packages in [TIME]
+    Installed 7 packages in [TIME]
+     + albatross==0.1.0 (from file://[TEMP_DIR]/albatross-virtual-workspace/packages/albatross)
+     + anyio==4.3.0
+     + bird-feeder==1.0.0 (from file://[TEMP_DIR]/albatross-virtual-workspace/packages/bird-feeder)
+     + idna==3.6
+     + seeds==1.0.0 (from file://[TEMP_DIR]/albatross-virtual-workspace/packages/seeds)
+     + sniffio==1.3.1
+     + tqdm==4.66.2
+    "###
+    );
+
+    Ok(())
+}
+
 /// Check that `uv run --package` works in a root workspace.
 #[test]
 fn test_uv_run_with_package_root_workspace() -> Result<()> {
@@ -508,3 +570,123 @@ fn workspace_lock_idempotence_virtual_workspace() -> Result<()> {
     )?;
     Ok(())
 }
+
+/// Extract just the sources from the lock file, to test path resolution.
+#[derive(Deserialize, Serialize)]
+struct SourceLock {
+    distribution: Vec<Distribution>,
+}
+
+impl SourceLock {
+    fn sources(self) -> BTreeMap<String, toml::Value> {
+        self.distribution
+            .into_iter()
+            .map(|distribution| (distribution.name, distribution.source))
+            .collect()
+    }
+}
+
+#[derive(Deserialize, Serialize)]
+struct Distribution {
+    name: String,
+    source: toml::Value,
+}
+
+/// Test path dependencies from one workspace into another.
+///
+/// We have a main workspace with packages `a` and `b`, and a second workspace with `c`, `d` and
+/// `e`. We have `a -> b`, `b -> c`, `c -> d`. `e` should not be installed.
+#[test]
+fn workspace_to_workspace_paths_dependencies() -> Result<()> {
+    let context = TestContext::new("3.12");
+
+    // Build the main workspace ...
+    let main_workspace = context.temp_dir.child("main-workspace");
+    main_workspace
+        .child("pyproject.toml")
+        .write_str(indoc! {r#"
+        [tool.uv.workspace]
+        members = ["packages/*"]
+    "#})?;
+
+    // ... with a  ...
+    let deps = indoc! {r#"
+        dependencies = ["b"]
+
+        [tool.uv.sources]
+        b = { workspace = true }
+    "#};
+    make_project(&main_workspace.join("packages").join("a"), "a", deps)?;
+
+    // ... and b.
+    let deps = indoc! {r#"
+        dependencies = ["c"]
+
+        [tool.uv.sources]
+        c = { path = "../../../other-workspace/packages/c", editable = true }
+    "#};
+    make_project(&main_workspace.join("packages").join("b"), "b", deps)?;
+
+    // Build the second workspace ...
+    let other_workspace = context.temp_dir.child("other-workspace");
+    other_workspace
+        .child("pyproject.toml")
+        .write_str(indoc! {r#"
+        [tool.uv.workspace]
+        members = ["packages/*"]
+    "#})?;
+
+    // ... with c  ...
+    let deps = indoc! {r#"
+        dependencies = ["d"]
+
+        [tool.uv.sources]
+        d = { workspace = true }
+    "#};
+    make_project(&other_workspace.join("packages").join("c"), "c", deps)?;
+
+    // ... and d ...
+    let deps = indoc! {r"
+        dependencies = []
+    "};
+    make_project(&other_workspace.join("packages").join("d"), "d", deps)?;
+
+    // ... and e.
+    let deps = indoc! {r#"
+        dependencies = ["numpy>=2.0.0,<3"]
+    "#};
+    make_project(&other_workspace.join("packages").join("e"), "e", deps)?;
+
+    uv_snapshot!(context.filters(), context.lock().arg("--preview").current_dir(&main_workspace), @r###"
+    success: true
+    exit_code: 0
+    ----- stdout -----
+
+    ----- stderr -----
+    Using Python 3.12.[X] interpreter at: [PYTHON-3.12]
+    Resolved 4 packages in [TIME]
+    "###
+    );
+
+    let lock: SourceLock =
+        toml::from_str(&fs_err::read_to_string(main_workspace.join("uv.lock"))?)?;
+
+    assert_json_snapshot!(lock.sources(), @r###"
+    {
+      "a": {
+        "editable": "packages/a"
+      },
+      "b": {
+        "editable": "packages/b"
+      },
+      "c": {
+        "editable": "../other-workspace/packages/c"
+      },
+      "d": {
+        "editable": "../other-workspace/packages/d"
+      }
+    }
+    "###);
+
+    Ok(())
+}
